/**
 * Dasu Project
 *
 * File:         DasuClassLoader.java
 * RCS:          $Id: DasuClassLoader.java,v 1.0 Aug 9, 2012 10:17:23 AM mas939 Exp $
 * Description:  DasuClassLoader class (see below)
 * Author:       Mario Sanchez
 *               Northwestern Systems Research Group
 *               Department of Computer Science
 *               Northwestern University
 * Created:      Aug 9, 2012 at 10:17:23 AM
 * Language:     Java
 * Package:      edu.northwestern.dasu.sandbox
 * Status:       Experimental (Do Not Distribute)
 *
 * (C) Copyright 2012, Northwestern University, all rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */
package edu.northwestern.dasu.util;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.security.KeyStore;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.security.cert.CertificateException;
import java.security.cert.X509Certificate;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Properties;
import java.util.jar.JarEntry;
import java.util.jar.JarFile;
import java.util.zip.ZipEntry;

import edu.northwestern.dasu.security.JarVerifier;

/**
 * @author Mario Sanchez &lt;msanchez@eecs.northwestern.edu&gt;
 *
 * The DasuClassLoader class ...
 */

public class DasuClassLoader extends ClassLoader{

	public final static java.util.logging.Logger LOGGER = java.util.logging.Logger.getLogger("edu.northwestern.dasu");
	private String jarFile;
	private JarFile jar;
	
    public DasuClassLoader(ClassLoader parent, String jarFile) {
        super(parent);
        updateJarFile(jarFile);
    }
    
    public void updateJarFile(String file) {
    	jarFile = file;
    	
    	LOGGER.info("CLASSLOADER: DasuClassLoader: " + jarFile);
        try {
			jar = new JarFile(jarFile);
		} catch (Exception e) {
			LOGGER.severe("CLASSLOADER: could not open " + jarFile);
		}
    }

    public Class loadClass(String className) throws ClassNotFoundException {
    	
    	try {

    		// Step 1 -- Check for a previously loaded class        	
            Class cls = findLoadedClass(className);
            if (cls != null) {
               return cls;
            }

            //System.out.println("get: " + className);
			String clazz = FindClass.findClass(jarFile, className, false, false);
			//System.out.println("load: "+ clazz);
			if (clazz != null && !clazz.equals("") ){
				
				String[] classesFound = clazz.split("\n");
				
				String c = classesFound[classesFound.length-1];
				//System.out.println("inside: " + c);
				//System.out.println("inside: " + className);
				if ( c.contains(className.replace(".", File.separator))) {
					
			        LOGGER.info(" CLASSLOADER: DasuClassLoader: " + c);
					String classNameSuffix = c.replaceFirst(jar.getName() + File.separator, "");
					//System.out.println(rand + " class:*" + c + "*");
					//System.out.println(rand + " look inside jar:*" + classNameSuffix + "*");
					byte[] classData = getBytes(classNameSuffix);
					return defineClass(className, classData, 0, classData.length, null);
	
				}
			}
			
		} catch (Exception e2) {
			e2.printStackTrace();
		}
    	    	
    	return super.loadClass(className);
    	
//    	if ( !className.contains("edu.northwestern.dasu") || !className.contains("dynamic") )
//    		return super.loadClass(className);   	
//
//        try {
//        	
//            // Step 1 -- Check for a previously loaded class        	
//            Class cls = findLoadedClass(className);
//            if (cls != null) {
//               return cls;
//            }
//        	                    
//            String classNameSuffix = className.replace(".","/") + ".class";
//            InputStream is = null;
//            
//            try {
//            	System.out.println(classNameSuffix);
//	            JarEntry entry = jar.getJarEntry(classNameSuffix);	            
//            
//	            is = jar.getInputStream(entry);
//            }
//            catch (Exception e1) {                        
//            	//e1.printStackTrace();            	
//            	is = getResourceAsStream(classNameSuffix);
//            }
//            
//            LOGGER.info("CLASSLOADER: DasuClassLoader: " + className.replace(".","/") + ".class" + " " + className);
//            ByteArrayOutputStream byteStream = new ByteArrayOutputStream();
//            int nextValue = is.read();
//            while (-1 != nextValue) {
//                byteStream.write(nextValue);
//                nextValue = is.read();
//            }
//
//            byte[] classData = byteStream.toByteArray();
//            return defineClass(className, classData, 0, classData.length, null);
//
//        } catch (Exception e) {
//        	
//        	//e.printStackTrace();
//        	return super.loadClass(className);
//        }
        	
        
    }

    private byte[] getBytes(String classNameSuffix) throws IOException {
    	
        JarEntry entry = jar.getJarEntry(classNameSuffix);
        InputStream is = jar.getInputStream(entry);
        ByteArrayOutputStream byteStream = new ByteArrayOutputStream();
        int nextValue = is.read();
        while (-1 != nextValue) {
            byteStream.write(nextValue);
            nextValue = is.read();
        }

        byte[] classData = byteStream.toByteArray();
        
        return classData;
    }
    
    public static void main(String[] args) {
    	
    	System.out.println("\n\n");
    	
    	try {
    		
    		JarFile jf = new JarFile("/tmp/DasuUiRulesModules.jar");
    		
	   		 KeyStore ks = null;
	         ks = KeyStore.getInstance("JKS");
	         InputStream is = new FileInputStream("/Users/mas939/Desktop/dasu.jks");
	         ks.load(is, "fadazani".toCharArray());
	         
	         ArrayList<X509Certificate> certs = new ArrayList();
	         Enumeration<String> en = ks.aliases();
	         for (; en.hasMoreElements(); ) {
	             String alias = (String)en.nextElement();
	             if (ks.isCertificateEntry(alias)) {
	            	 certs.add((X509Certificate) ks.getCertificate(alias));
	             }
	         }

	         JarVerifier.verify(jf, certs.toArray(new X509Certificate[]{}));
	         //JarVerifier.verify(jf, new X509Certificate[]{(X509Certificate) ks.getCertificate("rulesuimodules.aqualab.cs.northwestern.edu")});	        
	         //JarVerifier.verify(jf, new X509Certificate[]{(X509Certificate) ks.getCertificate("reports.aqualab.cs.northwestern.edu")});
    		
	         jf = new JarFile("/tmp/DasuUiRulesModules.jar");
	         ZipEntry ze = jf.getEntry("probe.modules.properties");
	         is = jf.getInputStream(ze);
	         Properties prop = new Properties();
	         prop.load(is);
    		
	         for (String key : prop.stringPropertyNames()){
    			System.out.println(key + " " + prop.getProperty(key));
	         }
    		
    		
    	}
    		
    	catch (CertificateException e) {
    		e.printStackTrace();
    	} catch (KeyStoreException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		} catch (NoSuchAlgorithmException e) {
			e.printStackTrace();
		}
    	

    	
    	 
    }
}